import re
from .badge import Badge
from .colors import Color
from .styles import Style

# Package information
version = __version__ = "v1.11.2rc2"
__version_info__ = tuple(re.split("[.-]", __version__))
__title__ = "anybadge"
__summary__ = "A simple, flexible badge generator."
__uri__ = "https://github.com/jongracecox/anybadge"



# Create a dictionary of colors to make selections
# easier.
from enum import Enum


class Color(Enum):
    WHITE = "#FFFFFF"
    SILVER = "#C0C0C0"
    GRAY = "#808080"
    BLACK = "#000000"
    RED = "#E05D44"
    BRIGHT_RED = "#FF0000"
    MAROON = "#800000"
    OLIVE = "#808000"
    LIME = "#00FF00"
    BRIGHT_YELLOW = "#FFFF00"
    YELLOW = "#DFB317"
    GREEN = "#4C1"
    YELLOW_GREEN = "#A4A61D"
    AQUA = "#00FFFF"
    TEAL = "#008080"
    BLUE = "#0000FF"
    NAVY = "#000080"
    FUCHSIA = "#FF00FF"
    PURPLE = "#800080"
    ORANGE = "#FE7D37"
    LIGHT_GREY = "#9F9F9F"

    def __lt__(self, other):
        return self.name < other.name



import argparse
import sys
import textwrap

from anybadge.styles import Style
from anybadge.templates import get_template
from . import config
from .badge import Badge


def parse_args(args):
    """Parse the command line arguments."""
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=textwrap.dedent(
            """\
Command line utility to generate .svg badges.

This utility can be used to generate .svg badge images, using configurable
thresholds for coloring.  Values can be passed as string, integer or floating
point.  The type will be detected automatically.

Running the utility with a --file option will result in the .svg image being
written to file.  Without the --file option the .svg file content will be
written to stdout, so can be redirected to a file.

Some thresholds have been built in to save time.  To use these thresholds you
can simply specify the template name instead of threshold value/color pairs.

examples:

    Here are some usage specific examples that may save time on defining
    thresholds.

    Pylint
        anybadge.py --value=2.22 --file=pylint.svg pylint

        anybadge.py --label=pylint --value=2.22 --file=pylint.svg \\
          2=red 4=orange 8=yellow 10=green

    Coverage
        anybadge.py --value=65 --file=coverage.svg coverage

        anybadge.py --label=coverage --value=65 --suffix='%%' --file=coverage.svg \\
          50=red 60=orange 80=yellow 100=green

    CI Pipeline
        anybadge.py --label=pipeline --value=passing --file=pipeline.svg \\
          passing=green failing=red

"""
        ),
    )
    parser.add_argument("-l", "--label", type=str, help="The badge label.")
    parser.add_argument(
        "-v", "--value", type=str, help="The badge value.", required=True
    )
    parser.add_argument(
        "-m",
        "--value-format",
        type=str,
        default=None,
        help='Formatting string for value (e.g. "%%.2f" for 2dp floats)',
    )
    parser.add_argument(
        "-c",
        "--color",
        type=str,
        help="For fixed color badges use --color to specify the badge color.",
        default=config.DEFAULT_COLOR,
    )
    parser.add_argument(
        "-p", "--prefix", type=str, help="Optional prefix for value.", default=""
    )
    parser.add_argument(
        "-s", "--suffix", type=str, help="Optional suffix for value.", default=""
    )
    parser.add_argument(
        "-d",
        "--padding",
        type=int,
        help="Number of characters to pad on either side of the badge text.",
        default=config.NUM_PADDING_CHARS,
    )
    parser.add_argument(
        "-lp",
        "--label-padding",
        type=int,
        help="Number of characters to pad on either side of the badge label.",
        default=None,
    )
    parser.add_argument(
        "-vp",
        "--value-padding",
        type=int,
        help="Number of characters to pad on either side of the badge value.",
        default=None,
    )
    parser.add_argument(
        "-n",
        "--font",
        type=str,
        help="Font name.  Supported fonts: "
        ",".join(['"%s"' % x for x in config.FONT_WIDTHS.keys()]),
        default=config.DEFAULT_FONT,
    )
    parser.add_argument(
        "-z",
        "--font-size",
        type=int,
        help="Font size.",
        default=config.DEFAULT_FONT_SIZE,
    )
    parser.add_argument(
        "-t",
        "--template",
        type=str,
        help="Location of alternative template .svg file.",
        default=get_template("default"),
    )
    parser.add_argument(
        "-st",
        "--style",
        type=str,
        help="Alternative style of badge to create. Valid "
        'values are "gitlab-scoped", "default". This '
        "overrides any templates passed using --template.",
    )
    parser.add_argument(
        "-u",
        "--use-max",
        action="store_true",
        help="Use the maximum threshold color when the value exceeds the "
        "maximum threshold.",
    )
    parser.add_argument("-f", "--file", type=str, help="Output file location.")
    parser.add_argument(
        "-o",
        "--overwrite",
        action="store_true",
        help="Overwrite output file if it already exists.",
    )
    parser.add_argument(
        "-r",
        "--text-color",
        type=str,
        help="Text color. Single value affects both label"
        "and value colors.  A comma separated pair "
        "affects label and value text respectively.",
        default=config.DEFAULT_TEXT_COLOR,
    )
    parser.add_argument(
        "-e",
        "--semver",
        action="store_true",
        default=False,
        help="Treat value and thresholds as semantic versions.",
    )
    parser.add_argument(
        "args",
        nargs=argparse.REMAINDER,
        help="Pairs of <upper>=<color>. "
        "For example 2=red 4=orange 6=yellow 8=good. "
        'Read this as "Less than 2 = red, less than 4 = orange...".',
    )
    return parser.parse_args(args)


def main(args=None):
    """Generate a badge based on command line arguments."""

    # Args may be sent from command line of as args directly.
    if not args:
        args = sys.argv[1:]

    # Parse command line arguments
    args = parse_args(args)

    label = args.label
    threshold_text = args.args
    suffix = args.suffix

    # Check whether thresholds were sent as one word, and is in the
    # list of available styles.  If so, swap in the style.
    if len(args.args) == 1 and Style.exists(args.args[0].upper()):
        style_name = args.args[0].upper()
        style = Style[style_name]
        threshold_text = style.threshold.split(" ")
        if not args.label and style.label:
            label = style.label
        if not args.suffix and style.suffix:
            suffix = style.suffix

    if not label:
        raise ValueError("Label has not been set.  Please use --label argument.")

    # Create threshold list from args
    threshold_list = [x.split("=") for x in threshold_text]
    threshold_dict = {x[0]: x[1] for x in threshold_list}

    # Create badge object
    badge = Badge(
        label,
        args.value,
        value_prefix=args.prefix,
        value_suffix=suffix,
        default_color=args.color,
        num_padding_chars=args.padding,
        num_label_padding_chars=args.label_padding,
        num_value_padding_chars=args.value_padding,
        font_name=args.font,
        font_size=args.font_size,
        template=args.template,
        style=args.style,
        use_max_when_value_exceeds=args.use_max,
        thresholds=threshold_dict,
        value_format=args.value_format,
        text_color=args.text_color,
        semver=args.semver,
    )

    if args.file:
        # Write badge SVG to file
        badge.write_badge(args.file, overwrite=args.overwrite)
    else:
        print(badge.badge_svg_text)


if __name__ == "__main__":
    main()



import logging


DEFAULT_SERVER_PORT: int = 8000
DEFAULT_SERVER_LISTEN_ADDRESS: str = "localhost"
DEFAULT_LOGGING_LEVEL = logging.INFO

SERVER_PORT: int = DEFAULT_SERVER_PORT
SERVER_LISTEN_ADDRESS: str = DEFAULT_SERVER_LISTEN_ADDRESS



import argparse
import logging
from http.server import HTTPServer
from os import environ

from anybadge.server.request_handler import AnyBadgeHTTPRequestHandler
from anybadge.server import config

logger = logging.getLogger(__name__)


def run(listen_address: str = None, port: int = None):
    """Run a persistent webserver."""
    if not listen_address:
        listen_address = config.DEFAULT_SERVER_LISTEN_ADDRESS

    if not port:
        port = config.DEFAULT_SERVER_PORT

    server_address: Tuple[str, int] = (listen_address, port)  # type: ignore

    httpd = HTTPServer(server_address, AnyBadgeHTTPRequestHandler)
    logger.info("Serving at: http://%s:%s" % server_address)

    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        logger.info("Received keyboard interrupt. Shutting down...")


def parse_args() -> argparse.Namespace:
    """Parse command line arguments."""
    logger.debug("Parsing command line arguments.")
    parser = argparse.ArgumentParser(description="Run an anybadge server.")
    parser.add_argument(
        "-p",
        "--port",
        type=int,
        default=config.DEFAULT_SERVER_PORT,
        help=f"Server port number.  Default is {config.DEFAULT_SERVER_PORT}. This can also be set via an environment "
        "variable called ``ANYBADGE_PORT``.",
    )
    parser.add_argument(
        "-l",
        "--listen-address",
        type=str,
        default=config.DEFAULT_SERVER_LISTEN_ADDRESS,
        help=f"Server listen address.  Default is {config.DEFAULT_SERVER_LISTEN_ADDRESS}. This can also be set via an "
        f"environment variable called ``ANYBADGE_LISTEN_ADDRESS``.",
    )
    parser.add_argument(
        "-d", "--debug", action="store_true", help="Enable debug logging."
    )
    return parser.parse_args()


def main():
    """Run server."""

    # Check for environment variables
    if "ANYBADGE_PORT" in environ:
        config.DEFAULT_SERVER_PORT = environ["ANYBADGE_PORT"]

    if "ANYBADGE_LISTEN_ADDRESS" in environ:
        config.DEFAULT_SERVER_LISTEN_ADDRESS = environ["ANYBADGE_LISTEN_ADDRESS"]

    if "ANYBADGE_LOG_LEVEL" in environ:
        config.DEFAULT_LOGGING_LEVEL = logging.getLevelName(
            environ["ANYBADGE_LOG_LEVEL"]
        )

    # Parse command line args
    args: argparse.Namespace = parse_args()

    # Set logging level
    logging_level = config.DEFAULT_LOGGING_LEVEL
    if args.debug:
        logging_level = logging.DEBUG

    logging.basicConfig(
        format="%(asctime)-15s %(levelname)s:%(filename)s(%(lineno)d):%(funcName)s: %(message)s",
        level=logging_level,
    )
    logger.info("Starting up anybadge server.")

    run(listen_address=args.listen_address, port=args.port)


if __name__ == "__main__":
    main()



"""Templates package."""
import pkgutil

from anybadge.exceptions import UnknownBadgeTemplate


def get_template(name: str) -> str:
    """Get a template by name.

    Examples:

        >>> get_template('default')  # doctest: +ELLIPSIS
        '<?xml version="1.0" encoding="UTF-8...

    """
    try:
        data = pkgutil.get_data(__name__, name + ".svg")
    except FileNotFoundError as e:
        raise UnknownBadgeTemplate from e

    if not data:
        raise UnknownBadgeTemplate

    return data.decode("utf-8")



import os
from collections import OrderedDict
from pathlib import Path
from typing import Dict, Type, Optional, Union

from . import config
from .colors import Color
from .exceptions import UnknownBadgeTemplate

from .helpers import _get_approx_string_width


# Try and obtain packaging package to support version comparison.
from .templates import get_template

from packaging.version import Version


class Badge:
    """
    Badge class used to generate badges.

    Args:
        label(str): Badge label text.
        value(str): Badge value text.
        font_name(str, optional): Name of font to use.
        font_size(int, optional): Font size.
        num_padding_chars(float, optional): Number of padding characters to use to give extra
            space around text.
        num_label_padding_chars(float, optional): Number of padding characters to use to give extra
            space around label text.
        num_value_padding_chars(float, optional): Number of padding characters to use to give extra
            space around value text.
        template(str, optional): String containing the SVG template.  This should be valid SVG
            file content with place holders for variables to be populated during rendering.
        style(str, optional): Style of badge to create. This will make anybadge render a badge in a
            different style. Valid values are "gitlab-scoped", "default". Default is "default".
        value_prefix(str, optional): Prefix to be placed before value.
        value_suffix(str, optional): Suffix to be placed after value.
        thresholds(dict, optional): A dictionary containing thresholds used to select badge
            color based on the badge value.
        default_color(str, optional): Badge color as a name or as an HTML color code.
        use_max_when_value_exceeds(bool, optional): Choose whether to use the maximum threshold
            value when the badge value exceeds the top threshold.  Default is True.
        value_format(str, optional) String with formatting to be used to format the value text.
        text_color(str, optional): Text color as a name or as an HTML color code.
        semver(bool, optional): Used to indicate that the value is a semantic version number.

    Examples:

        Create a simple green badge:

        >>> badge = Badge('label', 123, default_color='green')

        Write a badge to file, overwriting any existing file:

        >>> badge = Badge('label', 123, default_color='green')
        >>> badge.write_badge('demo.svg', overwrite=True)

        Here are a number of examples showing thresholds, since there
        are certain situations that may not be obvious:

        >>> badge = Badge('pipeline', 'passing', thresholds={'passing': 'green', 'failing': 'red'})
        >>> badge.badge_color
        'green'

        2.32 is not <2
        2.32 is < 4, so 2.32 yields orange
        >>> badge = Badge('pylint', 2.32, thresholds={2: 'red',
        ...                                           4: 'orange',
        ...                                           8: 'yellow',
        ...                                           10: 'green'})
        >>> badge.badge_color
        'orange'

        8 is not <8
        8 is <4, so 8 yields orange
        >>> badge = Badge('pylint', 8, thresholds={2: 'red',
        ...                                        4: 'orange',
        ...                                        8: 'yellow',
        ...                                        10: 'green'})
        >>> badge.badge_color
        'green'

        10 is not <8, but use_max_when_value_exceeds defaults to
        True, so 10 yields green
        >>> badge = Badge('pylint', 11, thresholds={2: 'red',
        ...                                         4: 'orange',
        ...                                         8: 'yellow',
        ...                                         10: 'green'})
        >>> badge.badge_color
        'green'

        11 is not <10, and use_max_when_value_exceeds is set to
        False, so 11 yields the default color '#4c1'
        >>> badge = Badge('pylint', 11, use_max_when_value_exceeds=False,
        ...               thresholds={2: 'red', 4: 'orange', 8: 'yellow',
        ...                           10: 'green'})
        >>> badge.badge_color
        '#4c1'
    """

    #: Singleton variable to track current max mask_id. This is used by _get_next_mask_str class method.
    mask_id: int

    def __init__(
        self,
        label,
        value,
        font_name: str = None,
        font_size: int = None,
        num_padding_chars: int = None,
        num_label_padding_chars: float = None,
        num_value_padding_chars: float = None,
        template: Union[Path, str] = None,
        style: str = None,
        value_prefix: str = "",
        value_suffix: str = "",
        thresholds: Optional[Dict[float, str]] = None,
        default_color: str = None,
        use_max_when_value_exceeds: bool = True,
        value_format: str = None,
        text_color: str = None,
        semver: bool = False,
    ):
        """Constructor for Badge class."""
        # Set defaults if values were not passed
        if not font_name:
            font_name = config.DEFAULT_FONT
        if not font_size:
            font_size = config.DEFAULT_FONT_SIZE
        if num_label_padding_chars is None:
            if num_padding_chars is None:
                num_label_padding_chars = config.NUM_PADDING_CHARS
            else:
                num_label_padding_chars = num_padding_chars
        if num_value_padding_chars is None:
            if num_padding_chars is None:
                num_value_padding_chars = config.NUM_PADDING_CHARS
            else:
                num_value_padding_chars = num_padding_chars
        if isinstance(template, Path):
            template = str(template)
        if not template:
            template = get_template("default")
        if style not in ["gitlab-scoped"]:
            style = "default"
        if not default_color:
            default_color = config.DEFAULT_COLOR
        if not text_color:
            text_color = config.DEFAULT_TEXT_COLOR

        self.label = label
        self.value = value

        self.value_is_version = semver

        self.value_format = value_format
        if value_format:
            value_text = str(value_format % self.value_type(value))
        else:
            value_text = str(self.value_type(value))
        self.value_prefix = value_prefix
        self.value_suffix = value_suffix
        self.value_text = value_prefix + value_text + value_suffix

        if font_name not in config.FONT_WIDTHS:
            raise ValueError(
                'Font name "%s" not found. '
                "Available fonts: %s"
                % (font_name, ", ".join(config.FONT_WIDTHS.keys()))
            )
        self.font_name = font_name
        self.font_size = font_size
        self.num_label_padding_chars = num_label_padding_chars
        self.num_value_padding_chars = num_value_padding_chars
        self.template = template
        self.style = style
        self.thresholds = thresholds
        self.default_color = default_color

        # text_color can be passed as a single value or a pair of comma delimited values
        self.text_color = text_color
        text_colors = text_color.split(",")
        self.label_text_color = text_colors[0]
        self.value_text_color = text_colors[0]
        if len(text_colors) > 1:
            self.value_text_color = text_colors[1]

        self.use_max_when_value_exceeds = use_max_when_value_exceeds
        self.mask_str = self.__class__._get_next_mask_str()

    def __repr__(self) -> str:
        """Return a representation of the Badge object instance.

        The output of the __repr__ function could be used to recreate the current object.

        Examples:

            >>> badge = Badge('example', '123.456')
            >>> repr(badge)
            "Badge('example', '123.456')"

            >>> badge = Badge('example', '123.456', value_suffix='TB')
            >>> repr(badge)
            "Badge('example', '123.456', value_suffix='TB')"

            >>> badge = Badge('example', '123.456', text_color='#111111', value_suffix='TB')
            >>> repr(badge)
            "Badge('example', '123.456', value_suffix='TB', text_color='#111111')"

            >>> badge = Badge('example', '123', num_padding_chars=5)
            >>> repr(badge)
            "Badge('example', '123', num_padding_chars=5)"

            >>> badge = Badge('example', '123', num_label_padding_chars=5)
            >>> repr(badge)
            "Badge('example', '123', num_label_padding_chars=5)"

            >>> badge = Badge('example', '123', num_label_padding_chars=5, num_value_padding_chars=6,
            ...               template='template.svg', value_prefix='$', thresholds={10: 'green', 30: 'red'},
            ...               default_color='red', use_max_when_value_exceeds=False, value_format="%s m/s")
            >>> repr(badge)
            "Badge('example', '123', num_label_padding_chars=5, num_value_padding_chars=6, template='template.svg', value_prefix='$', thresholds={10: 'green', 30: 'red'}, default_color='red', use_max_when_value_exceeds=False, value_format='%s m/s')"

        """
        optional_args = ""
        if self.font_name != config.DEFAULT_FONT:
            optional_args += ", font_name=%s" % repr(self.font_name)
        if self.font_size != config.DEFAULT_FONT_SIZE:
            optional_args += ", font_size=%s" % repr(self.font_size)
        if self.num_label_padding_chars == self.num_value_padding_chars:
            if self.num_label_padding_chars != config.NUM_PADDING_CHARS:
                optional_args += ", num_padding_chars=%s" % repr(
                    self.num_label_padding_chars
                )
        else:
            if self.num_label_padding_chars != config.NUM_PADDING_CHARS:
                optional_args += ", num_label_padding_chars=%s" % repr(
                    self.num_label_padding_chars
                )
            if self.num_value_padding_chars != config.NUM_PADDING_CHARS:
                optional_args += ", num_value_padding_chars=%s" % repr(
                    self.num_value_padding_chars
                )
        if self.template != get_template("default"):
            optional_args += ", template=%s" % repr(self.template)
        if self.style != "default":
            optional_args += ", style=%s" % repr(self.style)
        if self.value_prefix != "":
            optional_args += ", value_prefix=%s" % repr(self.value_prefix)
        if self.value_suffix != "":
            optional_args += ", value_suffix=%s" % repr(self.value_suffix)
        if self.thresholds:
            optional_args += ", thresholds=%s" % repr(self.thresholds)
        if self.default_color != config.DEFAULT_COLOR:
            optional_args += ", default_color=%s" % repr(self.default_color)
        if not self.use_max_when_value_exceeds:
            optional_args += ", use_max_when_value_exceeds=%s" % repr(
                self.use_max_when_value_exceeds
            )
        if self.value_format:
            optional_args += ", value_format=%s" % repr(self.value_format)
        if self.text_color != config.DEFAULT_TEXT_COLOR:
            optional_args += ", text_color=%s" % repr(self.text_color)

        return "%s(%s, %s%s)" % (
            self.__class__.__name__,
            repr(self.label),
            repr(self.value),
            optional_args,
        )

    def _repr_svg_(self) -> str:
        """Return SVG representation when used inside Jupyter notebook cells.

        This will render the SVG immediately inside a notebook cell when creating
        a Badge instance without assigning it to an identifier.
        """
        return self.badge_svg_text

    @classmethod
    def _get_next_mask_str(cls) -> str:
        """Return a new mask ID from a singleton sequence maintained on the class.

        Returns: str
        """
        if not hasattr(cls, "mask_id"):
            cls.mask_id = 0

        cls.mask_id += 1

        return config.MASK_ID_PREFIX + str(cls.mask_id)

    def _get_svg_template(self) -> str:
        """Return the correct SVG template to render, based on the style and template
        that have been set

        Returns: str
        """
        if self.style == "gitlab-scoped":
            return get_template("gitlab_scoped")

        # Identify whether template is a file or the actual template text

        if len(self.template.split("\n")) == 1:
            try:
                return get_template(self.template)
            except UnknownBadgeTemplate:
                pass

            with open(self.template, mode="r") as file_handle:
                return file_handle.read()
        else:
            return self.template

    @property
    def semver_version(self) -> Version:
        """The semantic version represented by the value string.

        Returns: Version
        """
        return Version(self.value)

    @property
    def semver_thresholds(self) -> Optional[OrderedDict]:
        """Thresholds as a dict using Version as keys."""
        # Version is not a hashable type, so can't be used to create an
        # ordered dict directly. First we need to create an ordered list of keys
        if not self.thresholds:
            return None

        ordered_keys = sorted(self.thresholds.keys(), key=Version)
        return OrderedDict((key, self.thresholds[key]) for key in ordered_keys)

    @property
    def float_thresholds(self) -> Optional[Dict[float, str]]:
        """Thresholds as a dict using floats as keys."""
        if not self.thresholds:
            return None
        return {float(k): v for k, v in self.thresholds.items()}

    @property
    def value_is_float(self) -> bool:
        """Identify whether the value text is a float.

        Returns: bool
        """

        # If the value is an int then it should not be considered a float.
        # We need to check this first before we check whether it is a float because the
        # float check also returns True for an int string.
        if self.value_is_int:
            return False

        try:
            _ = float(self.value)
        except (ValueError, TypeError):
            return False
        else:
            return True

    @property
    def value_is_int(self) -> bool:
        """Identify whether the value text is an int.

        Returns: bool
        """
        try:
            a = float(self.value)
            b = int(self.value)
        except (ValueError, TypeError):
            return False
        else:
            return a == b

    @property
    def value_type(self) -> Type:
        """The Python type associated with the value.

        Returns: type
        """
        if self.value_is_version:
            return Version
        if self.value_is_float:
            return float
        elif self.value_is_int:
            return int
        else:
            return str

    @property
    def label_width(self) -> int:
        """The SVG width of the label text.

        ┌───────────────────────────┬────────────────────────────────┐
        │                           │                                │
        │        Label text         │             Value              │
        │                           │                                │
        └───────────────────────────┴────────────────────────────────┘
                 ◀────────▶
                 label_width

        Returns: int
        """
        return int(
            self.get_text_width(str(self.label))
            + (2.0 * self.num_label_padding_chars * self.font_width)
        )

    @property
    def value_width(self) -> int:
        """The SVG width of the value text.

        ┌───────────────────────────┬────────────────────────────────┐
        │                           │                                │
        │        Label text         │           Value text           │
        │                           │                                │
        └───────────────────────────┴────────────────────────────────┘
                                                ◀────────▶
                                                value_width

        Returns: int
        """
        return int(
            self.get_text_width(str(self.value_text))
            + (2.0 * self.num_value_padding_chars * self.font_width)
        )

    @property
    def value_box_width(self) -> int:
        """The SVG width of the value text box.

        ┌───────────────────────────┬────────────────────────────────┐
        │                           │                                │
        │        Label text         │           Value text           │
        │                           │                                │
        └───────────────────────────┴────────────────────────────────┘
                                    ◀────────────────────────────────▶
                                             value_box_width

        Returns: int
        """
        return self.value_width - 9

    @property
    def font_width(self) -> int:
        """Return the width multiplier for a font.

        Returns:
            int: Maximum pixel width of badges selected font.

        Example:

            >>> Badge(label='x', value='1').font_width
            10
        """
        return config.FONT_WIDTHS[self.font_name][self.font_size]

    @property
    def color_split_position(self) -> int:
        """The SVG x position where the color split should occur.

                                     Split
                                       │
                                    ┌──┘
                                    ▼
        ┌───────────────────────────┬────────────────────────────────┐
        │                           │                                │
        │        Label text         │           Value text           │
        │                           │                                │
        └───────────────────────────┴────────────────────────────────┘
        ◀───────────────────────────▶
               color_split_pos

        Returns: int
        """
        return self.badge_width - self.value_width

    @property
    def label_anchor(self) -> float:
        """The SVG x position of the middle anchor for the label text.

                    Middle of
                      label
                     ┌──┘
                     ▼
        ┌───────────────────────────┬────────────────────────────────┐
        │                           │                                │
        │        Label text         │           Value text           │
        │                           │                                │
        └───────────────────────────┴────────────────────────────────┘
        ◀───────────▶
         label_anchor

        Returns: float
        """
        return self.color_split_position / 2

    @property
    def value_anchor(self) -> float:
        """The SVG x position of the middle anchor for the value text.

                                                   Middle of
                                                     value
                                                    ┌──┘
                                                    ▼
        ┌───────────────────────────┬────────────────────────────────┐
        │                           │                                │
        │        Label text         │           Value text           │
        │                           │                                │
        └───────────────────────────┴────────────────────────────────┘
        ◀──────────────────────────────────────────▶
                                               value_anchor

        Returns: float
        """
        return self.color_split_position + (
            (self.badge_width - self.color_split_position) / 2
        )

    @property
    def label_anchor_shadow(self) -> float:
        """The SVG x position of the label shadow anchor.

        The shadow for the label will appear behind the label.

              ┌ Text ──────────────────┐
              │                        │
              │                        ├────┐
              │                        │    │
              └────┬───────────────────┘    │
                   │                        │
                   └───────────── Shadow ───┘

        The label_anchor_shadow is the distance from left to center of shadow:

        ┌─────────────────────────────┬─────────────────────────────────┐
        │     ┌────────────┐          │                                 │
        │     │            ├─┐        │           Value text            │
        │     └─┬──────────┘ │        │                                 │
        │       └────────────┘        │                                 │
        └─────────────────────────────┴─────────────────────────────────┘
        ◀─────────────▶
        label_anchor_shadow

        Returns: float
        """
        return self.label_anchor + 1

    @property
    def value_anchor_shadow(self) -> float:
        """The SVG x position of the value shadow anchor.

        ┌─────────────────────────────┬─────────────────────────────────┐
        │                             │        ┌────────────┐           │
        │         Label text          │        │            ├─┐         │
        │                             │        └─┬──────────┘ │         │
        │                             │          └────────────┘         │
        └─────────────────────────────┴─────────────────────────────────┘
        ◀───────────────────────────────────────────────▶
                                               value_anchor_shadow
        Returns: float
        """
        return self.value_anchor + 1

    @property
    def badge_width(self) -> int:
        """The total width of badge.

        ┌───────────────────────────┬────────────────────────────────┐
        │                           │                                │
        │        Label text         │           Value text           │
        │                           │                                │
        └───────────────────────────┴────────────────────────────────┘
        ◀───────────────────────────────────────────────────────────▶
                                 badge_width

        Returns: int

        Examples:

            >>> badge = Badge('pylint', '5')
            >>> badge.badge_width
            61
        """
        return self.label_width + self.value_width

    @property
    def arc_start(self) -> int:
        """The position where the arc on the arc should start.

        Returns: int

        Examples:

            >>> badge = Badge('pylint', '5')
            >>> badge.arc_start
            51
        """
        return self.badge_width - 10

    @property
    def badge_svg_text(self) -> str:
        """The badge SVG text.

        Returns: str
        """

        badge_text = self._get_svg_template()

        return (
            badge_text.replace("{{ badge width }}", str(self.badge_width))
            .replace("{{ font name }}", self.font_name)
            .replace("{{ font size }}", str(self.font_size))
            .replace("{{ label }}", self.label)
            .replace("{{ value }}", self.value_text)
            .replace("{{ label anchor }}", str(self.label_anchor))
            .replace("{{ label anchor shadow }}", str(self.label_anchor_shadow))
            .replace("{{ value anchor }}", str(self.value_anchor))
            .replace("{{ value anchor shadow }}", str(self.value_anchor_shadow))
            .replace("{{ color }}", self.badge_color_code)
            .replace("{{ label text color }}", self.label_text_color)
            .replace("{{ value text color }}", self.value_text_color)
            .replace("{{ color split x }}", str(self.color_split_position))
            .replace("{{ value width }}", str(self.value_width))
            .replace("{{ mask id }}", self.mask_str)
            .replace("{{ value box width }}", str(self.value_box_width))
            .replace("{{ arc start }}", str(self.arc_start))
        )

    def __str__(self) -> str:
        """Return string representation of badge.

        This will return the badge SVG text.

        Returns: str

        Examples:

            >>> print(Badge('example', '123'))  # doctest: +ELLIPSIS
            <?xml version="1.0" encoding="UTF-8"?>
            ...
        """
        return self.badge_svg_text

    def get_text_width(self, text) -> int:
        """Return the width of text.

        Args:
            text(str): Text to get the pixel width of.

        Returns:
            int: Pixel width of the given text based on the badges selected font.

        This implementation assumes a fixed font of:

        font-family="DejaVu Sans,Verdana,Geneva,sans-serif" font-size="11"
        >>> badge = Badge('x', 1, font_name='DejaVu Sans,Verdana,Geneva,sans-serif', font_size=11)
        >>> badge.get_text_width('pylint')
        34
        """
        return _get_approx_string_width(text, self.font_width)

    @property
    def badge_color(self) -> str:
        """Badge color based on the configured thresholds.

        Returns: str"""
        # If no thresholds were passed then return the default color
        if not self.thresholds:
            return self.default_color

        if self.value_type == str:
            if self.value in self.thresholds:
                return self.thresholds[self.value]
            else:
                return self.default_color

        thresholds: Optional[Union[Dict[float, str], OrderedDict[float, str]]]

        # Set value and thresholds based on the value type. This will result in either
        # value and thresholds as floats or value and thresholds as semantic versions.
        if self.value_type == Version:
            value = self.semver_version
            thresholds = self.semver_thresholds
        else:
            value = float(self.value)
            thresholds = self.float_thresholds

        color = None

        if thresholds:

            # Convert the threshold dictionary into a sorted list of lists
            threshold_list = [[self.value_type(i[0]), i[1]] for i in thresholds.items()]
            threshold_list.sort(key=lambda x: x[0])

            for threshold, color in threshold_list:
                if value < threshold:
                    return color

        # If we drop out the top of the range then return the last max color
        if color and self.use_max_when_value_exceeds:
            return color
        else:
            return self.default_color

    @property
    def badge_color_code(self) -> str:
        """Return the color code for the badge.

        Returns: str

        Raises: ValueError when an invalid badge color is set.
        """
        color = self.badge_color

        if isinstance(color, Color):
            return color.value

        if color[0] == "#":
            return color

        color = color.upper()

        prefixes = ["BRIGHT", "YELLOW", "LIGHT"]

        try:
            return Color[color.upper()].value
        except KeyError:
            pass

        # For backward compatibility with old color names (that were lowercase and didn't
        # contain underscores) we will try to get the same color.

        for prefix in prefixes:
            if color.startswith(prefix) and color != prefix and "_" not in color:
                try:
                    return Color[color.replace(prefix, prefix + "_")].value
                except KeyError:
                    pass

        raise ValueError(
            'Invalid color code "%s". ' "Valid color codes are: %s",
            (color, ", ".join(list(Color.__members__.keys()))),
        )

    def write_badge(self, file_path: Union[str, Path], overwrite=False) -> None:
        """Write badge to file."""

        if isinstance(file_path, str):

            if file_path.endswith("/"):
                raise ValueError("File location may not be a directory.")

            file: Path = Path(file_path)
        else:
            file = file_path

        # Validate path (part 1)
        if file.is_dir():
            raise ValueError("File location may not be a directory.")

        # Ensure we're using a .svg extension
        file = file.with_suffix(".svg")

        # Validate path (part 2)
        if not overwrite and file.exists():
            raise RuntimeError('File "{}" already exists.'.format(file))

        with open(file, mode="w") as file_handle:
            file_handle.write(self.badge_svg_text)



# Set some defaults
from typing import Dict

DEFAULT_FONT: str = "DejaVu Sans,Verdana,Geneva,sans-serif"
DEFAULT_FONT_SIZE: int = 11
NUM_PADDING_CHARS: float = 0.5
DEFAULT_COLOR: str = "#4c1"
DEFAULT_TEXT_COLOR: str = "#fff"
MASK_ID_PREFIX: str = "anybadge_"

# Dictionary for looking up approx pixel widths of
# supported fonts and font sizes.
FONT_WIDTHS: Dict[str, Dict[int, int]] = {
    "DejaVu Sans,Verdana,Geneva,sans-serif": {
        10: 9,
        11: 10,
        12: 11,
    },
    "Arial, Helvetica, sans-serif": {
        11: 8,
    },
}



# Based on the following SO answer: https://stackoverflow.com/a/16008023/6252525
def _get_approx_string_width(text, font_width, fixed_width=False) -> int:
    """
    Get the approximate width of a string using a specific average font width.

    Args:
        text(str): Text string to calculate width of.
        font_width(int): Average width of font characters.
        fixed_width(bool): Indicates that the font is fixed width.

    Returns:
        int: Width of string in pixels.

    Examples:

        Call the function with a string and the maximum character width of the font you are using:

            >>> int(_get_approx_string_width('hello', 10))
            29

        This example shows the comparison of simplistic calculation based on a fixed width.
        Given a test string and a fixed font width of 10, we can calculate the width
        by multiplying the length and the font character with:

            >>> test_string = 'GOOGLE|ijkl'
            >>> _get_approx_string_width(test_string, 10, fixed_width=True)
            110

        Since some characters in the string are thinner than others we expect that the
        apporximate text width will be narrower than the fixed width calculation:

            >>> _get_approx_string_width(test_string, 10)
            77

    """
    if fixed_width:
        return len(text) * font_width

    size = 0.0

    # A dictionary containing percentages that relate to how wide
    # each character will be represented in a variable width font.
    # These percentages can be calculated using the ``_get_character_percentage_dict`` function.
    char_width_percentages = {
        "lij|' ": 40.0,
        "![]fI.,:;/\\t": 50.0,
        '`-(){}r"': 60.0,
        "*^zcsJkvxy": 70.0,
        "aebdhnopqug#$L+<>=?_~FZT0123456789": 70.0,
        "BSPEAKVXY&UwNRCHD": 70.0,
        "QGOMm%W@": 100.0,
    }

    for s in text:
        percentage = 100.0
        for k in char_width_percentages.keys():
            if s in k:
                percentage = char_width_percentages[k]
                break
        size += (percentage / 100.0) * float(font_width)

    return int(size)



#!/usr/bin/python
import os
import re
from setuptools import setup

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

# Attempt to get version number from TravisCI environment variable
version = os.environ.get("TRAVIS_TAG", default="0.0.0")

# Remove leading 'v'
version = re.sub("^v", "", version)

setup(
    name="anybadge",
    description="Simple, flexible badge generator for project badges.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    version=version,
    author="Jon Grace-Cox",
    author_email="jongracecox@gmail.com",
    packages=["anybadge", "anybadge.templates", "anybadge.server"],
    py_modules=["anybadge_server"],
    setup_requires=["setuptools", "wheel"],
    tests_require=[],
    install_requires=["packaging"],
    package_data={"anybadge": ["templates/*.svg"]},
    options={"bdist_wheel": {"universal": False}},
    python_requires=">=3.7",
    url="https://github.com/jongracecox/anybadge",
    entry_points={
        "console_scripts": [
            "anybadge=anybadge.cli:main",
            "anybadge-server=anybadge.server.cli:main",
        ],
    },
    classifiers=["License :: OSI Approved :: MIT License"],
)



import logging
import urllib.parse as urlparse
from http.server import BaseHTTPRequestHandler

from anybadge import Badge

logger = logging.getLogger(__name__)


class AnyBadgeHTTPRequestHandler(BaseHTTPRequestHandler):
    """Request handler for anybadge HTTP server."""

    def do_HEAD(self):
        logging.debug("Sending head.")
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()

    def do_GET(self):
        logging.debug("Handling get request.")
        self.do_HEAD()

        # Ignore request for favicon
        if self.path == "/favicon.ico":
            logging.debug("Ignoring favicon request.")
            return

        # Parse the URL query string
        parsed = urlparse.urlparse(self.path)
        url_query = urlparse.parse_qs(parsed.query)

        label = ""
        value = ""
        color = "green"

        # Extract the label and value portions
        if "label" in url_query:
            label = url_query["label"][0]

        if "value" in url_query:
            value = url_query["value"][0]

        logging.debug("Label: %s Value: %s", label, value)

        if label and value and color:
            logging.debug("All parameters present.")
            badge = Badge(label=label, value=value, default_color=color)
            for line in badge.badge_svg_text:
                self.wfile.write(str.encode(line))

        else:
            logging.debug("Not all parameters present.")

            self.wfile.write(b"<html><head><title>Anybadge Web Server.</title></head>")
            self.wfile.write(b"<body>")

            listen_host, listen_port = self.server.server_address

            help_text = f"""
                        <h1>Welcome to the Anybadge Web Server.</h1>

                        You are seeing this message because you haven't passed all the query parameters
                        to display a badge.

                        You need to pass at least a <b>label</b> and <b>value</b> parameter.

                        Here is an example:

                        <a href="http://{listen_host}:{listen_port}/?label=Project%20Awesomeness&value=110%">\
                        http://{listen_host}:{listen_port}/?label=Project%20Awesomeness&value=110%</a>
                        """

            for line in help_text.splitlines():
                self.wfile.write(str.encode("<p>%s</p>" % line))
            self.wfile.write(b"</body></html>")



from enum import Enum


# Define some templates that can be used for common badge types, saving
# from having to provide thresholds and labels each time.
class Style(Enum):
    """A style that can be used for common badge types."""

    PYLINT = ("default.svg", "2=red 4=orange 8=yellow 10=green", "pylint")
    COVERAGE = ("default.svg", "50=red 60=orange 80=yellow 100=green", "coverage", "%")

    def __init__(self, template, threshold, label, suffix=None):
        self.template = template
        self.threshold = threshold
        self.label = label
        self.suffix = suffix

    @classmethod
    def exists(cls, name: str) -> bool:
        """Test whether a style exists."""
        try:
            _ = cls[name]
            return True
        except KeyError:
            return False
